(function() {
  var VerticaDate, VerticaInterval, VerticaTime, padWithZeroes;
  padWithZeroes = function(str, length) {
    var res;
    res = "" + str;
    while (res.length < length) {
      res = "0" + res;
    }
    return res;
  };
  VerticaDate = (function() {
    function VerticaDate(year, month, day) {
      this.year = +year;
      this.month = +month;
      this.day = +day;
    }
    VerticaDate.prototype.toDate = function() {
      return new Date(this.year, this.month - 1, this.day);
    };
    VerticaDate.prototype.toString = function() {
      return "" + (padWithZeroes(this.year, 4)) + "-" + (padWithZeroes(this.month, 2)) + "-" + (padWithZeroes(this.day, 2));
    };
    VerticaDate.prototype.sqlQuoted = function() {
      return "'" + (this.toString()) + "'::date";
    };
    VerticaDate.prototype.toJSON = function() {
      return this.toString();
    };
    return VerticaDate;
  })();
  VerticaDate.fromStringBuffer = function(buffer) {
    var matches;
    if (matches = buffer.toString('ascii').match(/^(\d{4})-(\d{2})-(\d{2})$/)) {
      return new VerticaDate(matches[1], matches[2], matches[3]);
    } else {
      throw 'Invalid date format!';
    }
  };
  VerticaDate.fromDate = function(date) {
    return new VerticaDate(date.getFullYear(), date.getMonth() + 1, date.getDate());
  };
  VerticaTime = (function() {
    function VerticaTime(hour, minute, second) {
      this.hour = +hour;
      this.minute = +minute;
      this.second = +second;
    }
    VerticaTime.prototype.toString = function() {
      return "" + (padWithZeroes(this.hour, 2)) + ":" + (padWithZeroes(this.minute, 2)) + ":" + (padWithZeroes(this.second, 2));
    };
    VerticaTime.prototype.sqlQuoted = function() {
      return "'" + (this.toString()) + "'::time";
    };
    VerticaTime.prototype.toJSON = function() {
      return this.toString();
    };
    return VerticaTime;
  })();
  VerticaTime.fromStringBuffer = function(buffer) {
    var matches;
    if (matches = buffer.toString('ascii').match(/^(\d{2}):(\d{2}):(\d{2})$/)) {
      return new VerticaTime(matches[1], matches[2], matches[3]);
    } else {
      throw 'Invalid time format!';
    }
  };
  VerticaInterval = (function() {
    function VerticaInterval(days, hours, minutes, seconds) {
      if (days != null) {
        this.days = +days;
      }
      if (hours != null) {
        this.hours = +hours;
      }
      if (minutes != null) {
        this.minutes = +minutes;
      }
      if (seconds != null) {
        this.seconds = +seconds;
      }
    }
    VerticaInterval.prototype.inDays = function() {
      var days;
      days = 0;
      if (this.days) {
        days += this.days;
      }
      if (this.hours) {
        days += this.hours / 24;
      }
      if (this.minutes) {
        days += this.minutes / (24 * 60);
      }
      if (this.seconds) {
        return days += this.seconds / (24 * 60 / 60);
      }
    };
    VerticaInterval.prototype.inSeconds = function() {
      var seconds;
      seconds = 0;
      if (this.days) {
        seconds += this.days * 60 * 60 * 24;
      }
      if (this.hours) {
        seconds += this.hours * 60 * 60;
      }
      if (this.minutes) {
        seconds += this.minutes * 60;
      }
      if (this.seconds) {
        return seconds += this.seconds;
      }
    };
    VerticaInterval.prototype.inMilliseconds = function() {
      return this.inSeconds() * 1000;
    };
    VerticaInterval.prototype.inMicroseconds = function() {
      return this.inSeconds() * 1000000;
    };
    VerticaInterval.prototype.toJSON = function() {
      return {
        days: this.days,
        hours: this.hours,
        minutes: this.minutes,
        seconds: this.seconds
      };
    };
    VerticaInterval.prototype.sqlQuoted = function() {
      throw 'Not yet implemented';
    };
    return VerticaInterval;
  })();
  VerticaInterval.fromStringBuffer = function(buffer) {
    var matches;
    if (matches = buffer.toString('ascii').match(/^(\d+)?\s?(?:(\d{2}):(\d{2})(?::(\d{2}(?:\.\d+)?))?)?$/)) {
      return new VerticaInterval(matches[1], matches[2], matches[3], matches[4]);
    } else {
      throw 'Invalid interval format!';
    }
  };
  exports.Date = VerticaDate;
  exports.Time = VerticaTime;
  exports.Interval = VerticaInterval;
}).call(this);
