(function() {
  var Vertica, binaryDecoders, stringDecoders;
  Vertica = require('./vertica');
  stringDecoders = {
    string: function(value) {
      return value.toString();
    },
    integer: function(value) {
      return +value;
    },
    real: function(value) {
      return parseFloat(value);
    },
    numeric: function(value) {
      return parseFloat(value);
    },
    boolean: function(value) {
      return value.toString() === 't';
    },
    date: function(value) {
      return Vertica.Date.fromStringBuffer(value);
    },
    time: function(value) {
      return Vertica.Time.fromStringBuffer(value);
    },
    interval: function(value) {
      return Vertica.Interval.fromStringBuffer(value);
    },
    timestamp: function(value) {
      var matches, timestampRegexp, timezoneOffset, utc;
      timestampRegexp = /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})(\.\d{1,})?(?:([\+\-])(\d{2})(?:\:(\d{2}))?)?$/;
      if (matches = value.toString('ascii').match(timestampRegexp)) {
        utc = Date.UTC(+matches[1], +matches[2] - 1, +matches[3], +matches[4], +matches[5], +matches[6], Math.round(+matches[7] * 1000) || 0);
        if (matches[8]) {
          timezoneOffset = +matches[9] * 60 + (+matches[10] || 0);
          if (matches[8] === '-') {
            timezoneOffset = 0 - timezoneOffset;
          }
          utc -= timezoneOffset * 60 * 1000;
        } else if (Vertica.timezoneOffset) {
          utc -= Vertica.timezoneOffset;
        }
        return new Date(utc);
      } else {
        throw 'Invalid timestamp string returned';
      }
    },
    "default": function(value) {
      return value.toString();
    }
  };
  binaryDecoders = {
    "default": function(value) {
      throw 'Binary decoders not yet supported!';
    }
  };
  module.exports = {
    0: stringDecoders,
    1: binaryDecoders,
    'string': stringDecoders,
    'binary': binaryDecoders
  };
}).call(this);
